@@ -32,7 +32,7 @@ magenta=_magenta
 cyan=_cyan
 white=_white
 grey=_grey
-_version='N0Lib:0.0.5'
+_version='N0Lib:0.0.7'
 __N0Version__=''
 __SETVERSION__=False
 __DEBUG__=False 
@@ -40,6 +40,7 @@ __GRAPHIC__=False
 __dirChar__='/'  
 __WORKPATH__='./'
 __SHELLPATH__='./'
+__DEFAULT_STDOUT__=sys.stdout
 __FILTERMODE__='half'
 __COLORFILTER__='$n0p'
 __SYSTEM_VARS__={'DEBUG':'off','TIMEOUT':30,'BACKSPACE':263,'TAB':9,'FASTUP':337,'FASTDOWN':336,'FILTERMODE':'half','DEBUGLEVEL':1,'__SAFEMODE__':'on','__STDOUT__':'$n0p','FLUSHCHECK':'off'}
@@ -407,7 +408,7 @@ def N0HELP(_function='$n0p'):
         'debugOut':'与consoleOut完全一致的用法，除了它仅在DEBUG模式下才执行，以及一定不受滤镜影响。',
         'rainbowOut':'非常多彩的consoleOut。(输出者,输出内容,[输出者颜色],[循环颜色序列],[循环开始颜色],[循环结束颜色],[是否输出],[是否受滤镜影响]) 参数会造成什么影响自己实验一下吧(doge; 如果把输出关闭，rainbowOut也可以作为生成多彩染色字符串的工具。',
         'echo':'简单输出。(字符串,[颜色]) 在你打算将某程序改用N0Lib开发的版本时，可以将print简单替换为echo。也许偶尔简单的输出一下很方便，不过还是更建议使用consoleOut。',
-        'flowSay':'紧跟在上一次SAY的后面输出。(字符串,[颜色])',
+        'followSay':'紧跟在上一次SAY的后面输出。(字符串,[颜色])',
         'SAY':'在屏幕任意位置输出字符串。(输出横坐标,输出纵坐标,字符串,[颜色],[是否打印背景],[背景字符])',
         'realLength':'获取字符串显示长度。(字符串)',
         'verticalLine':'绘制竖线。(输出横坐标,输出纵坐标,打印字符,[颜色])',
@@ -619,6 +620,406 @@ def publicVar(publicVarName,_value='$n0p'):
             __PublicVarPool__[publicVarName]=str(_value)
         except:
             debugOut('publicVarSync',"$("+publicVarName+") synchronizes failed.")
+def ENTER(_x,_y,limitLength,color=_white,backStr='Please input.',backColor=_grey,keyList='$n0p',default='$n0p',history='$n0p',clear=True,hideChar='$n0p',maxLine=1,matchConfig='$n0p',__DEEP__=0,STDOUT='$n0p'):
+    _x,_y,limitLength,maxLine=_legalSize(_x,_y,limitLength,maxLine)
+    if STDOUT=='$n0p':
+        STDOUT=__SYSTEM_VARS__['__STDOUT__']
+    def saveHistory(_str):
+        if history!='$n0p':#保留历史记录
+                historyLength=len(history)
+                if _str!='':
+                    if historyLength>0:
+                        if _str!=history[historyLength-1]:
+                            history.append(_str)
+                    else:
+                        history.append(_str)
+    retStr,flags=_ENTERLooper(_x,_y,limitLength,color,backStr,backColor,keyList,default,history,clear,hideChar,maxLine,matchConfig,0,STDOUT=STDOUT)
+    if flags=='$enter':
+        saveHistory(retStr)
+        return retStr
+    else:
+        return "N0Shell doesn't know what happend.Maybe you should ask for N0P3."
+def _ENTERLooper(_x,_y,limitLength,color=_white,backStr='Please input.',backColor=_grey,keyList='$n0p',default='$n0p',history='$n0p',clear=True,hideChar='$n0p',maxLine=1,matchConfig='$n0p',__DEEP__=0,keyWords=list(),STDOUT='$n0p'):
+    def getSelection(_PreExcute):
+        if _PreExcute!='':
+            retSelect=_PreExcute[0]
+            _PreExcute=_PreExcute[1:]
+            #consoleOut('PreExcute',_PreExcute,'green','white')
+        else:
+            try:
+                retSelect=_consoleWin.get_wch()
+            except Exception as e:
+                #debugOut('N0SHELL_WARNING',str(e),_yellow,_white)
+                retSelect=curses.KEY_RIGHT
+        #echo(retSelect)
+        return _PreExcute,retSelect
+    def createPreExcute(_str):
+        if _str!='$n0p':
+            _PreExcute=''
+            while True:
+                if len(_str)>limitLength:
+                    _PreExcute+=_str[:limitLength]
+                    _str=_str[limitLength+1:]
+                    _PreExcute+='\\'+chr(10)
+                else:
+                    _PreExcute+=_str
+                    break
+            return _PreExcute
+        else:
+            return ''
+    def hideClear():
+        cursor.hide()
+        drawLine(_x,_y,' ',limitLength+1)
+        cursor.show()
+    def retHistory(_index,_history,up=True):
+        nonlocal indexX,retStr
+        if up:
+            if _index>0:
+                _index-=1
+        else:
+            if _index<len(_history):
+                _index+=1
+        try:
+            if _index!=len(_history):
+                _PreExcute=createPreExcute(_history[_index])
+            else:
+                _PreExcute=''
+            indexX=0
+            retStr=''
+            hideClear()
+            say(_x,_y,'')
+            return esc(_PreExcute,['\\']),_index
+        except:
+            pass
+    indexX=0
+    retStr=''
+    keyWord=''
+    defaultMatchConfig={
+        'Words':[],
+        'Paths':[],
+        'CurrentPath':False,
+        'Var':False,
+        'Command':False,
+        'PATH':False
+    }
+    if matchConfig!='$n0p':#匹配设置
+        matchConfig=patchDict(defaultMatchConfig,matchConfig)
+    PreExcute=createPreExcute(default)#预执行命令
+    historyIndex=len(history)
+    if keyList=='$n0p':
+        whiteList=False
+    else:
+        whiteList=True
+    if clear:
+        hideClear()#清空输入区
+    say(_x,_y,backStr,backColor)
+    say(_x,_y,'')
+    while True:
+        cursor.show()
+        PreExcute,selection=getSelection(PreExcute)
+        #consoleOut('press',chr(selection),_white,_green)
+        try:
+            ordSelection=ord(selection)
+        except:
+            ordSelection=263
+        if selection=='\\':
+            say(_x+indexX,_y,'\\',_grey)
+            PreExcute,selection=getSelection(PreExcute)
+            if selection == '\n':#换行
+                #retStr+=(limitLength-len(retStr))*' '#换行部分补空格
+                #indexX=limitLength
+                if __DEEP__<maxLine-1:
+                    enterRet,flags=_ENTERLooper(_x,_y+1,limitLength,color,'',_grey,keyList,PreExcute,history,clear,hideChar,maxLine,matchConfig,__DEEP__+1,keyWords,STDOUT)
+                    PreExcute=''
+                    #debugOut('ENTER','enterRet '+enterRet)
+                    if flags=='$del':#通过删除键返回
+                        #debugOut('ENTER','Delete!')
+                        say(_x+indexX,_y,' ')
+                        say(_x,_y,retStr,color)
+                        continue
+                    elif flags=='$up':#通过上键返回
+                        if __DEEP__>0:
+                            retStr+=enterRet
+                            hideClear()
+                            return retStr,'$up'
+                        else:
+                            PreExcute,historyIndex=retHistory(historyIndex,history)
+                        continue
+                    elif flags=='$down':#通过下键返回
+                        if __DEEP__>0:
+                            hideClear()
+                            return '','$down'
+                        else:
+                            PreExcute,historyIndex=retHistory(historyIndex,history,False)
+                        continue
+                    else:#通过ENTER返回
+                        retStr+=enterRet
+                        if clear:
+                            hideClear()
+                        return retStr,'$enter'
+                else:#达到最大深度
+                    say(_x+indexX,_y,' ')
+                    say(_x,_y,retStr,color)
+                    PreExcute=''
+                    continue
+            elif selection == '\\':#输入字符\
+                say(_x+indexX,_y,' ')
+                pass
+            else:
+                say(_x+indexX,_y,' ')
+                try:
+                    ordSelection=ord(selection)
+                except:
+                    ordSelection=263
+                pass
+        if selection == '\n':#ENTER
+            if clear:
+                hideClear()
+            cursor.hide()
+            return retStr,'$enter'
+        elif selection==curses.KEY_UP:#up
+            if __DEEP__>0:
+                hideClear()
+                return retStr,'$up'
+            else:
+                PreExcute,historyIndex=retHistory(historyIndex,history)
+                keyWord=''
+        elif selection==curses.KEY_DOWN:#down
+            if __DEEP__>0:
+                hideClear()
+                return '','$down'
+            else:
+                PreExcute,historyIndex=retHistory(historyIndex,history,False)
+                keyWord=''
+        elif selection==curses.KEY_LEFT:
+            pass
+        elif selection==curses.KEY_RIGHT:
+            pass
+        elif selection==410:#特殊的拖动产生的神秘字符
+            pass
+        elif ordSelection==27:
+            pass
+        elif ordSelection==__SYSTEM_VARS__['BACKSPACE']:#backspace
+            if indexX>0:
+                chrWidth=getWidth(retStr[-1])
+                indexX-=chrWidth
+                retStr=retStr[:-1]
+                if len(keyWord)>0:
+                    keyWord=keyWord[:-1]#tab补全用
+                else:
+                    try:
+                        keyWord=keyWords.pop()
+                    except:
+                        keyWord=''
+                say(_x+indexX,_y,' '*chrWidth,color)
+                if indexX>0:
+                    chrWidth=getWidth(retStr[-1])
+                    if hideChar=='$n0p':
+                        say(_x+indexX-chrWidth,_y,retStr[-1],color)#Unknow
+                    else:
+                        say(_x+indexX-chrWidth,_y,hideChar[0]*chrWidth,color)#Unknow
+                else:
+                    say(_x,_y,backStr,backColor)
+                    say(_x,_y,'')
+            else:
+                if __DEEP__>0:#在深层递归
+                    return '','$del'
+                else:
+                    pass
+        elif selection==chr(__SYSTEM_VARS__['TAB']):
+            if keyWord!='':
+                if matchConfig!='$n0p':
+                    matchItems=_matchThings(keyWord,matchConfig['Words'],matchConfig['Paths'],matchConfig['CurrentPath'],matchConfig['Var'],matchConfig['Command'],matchConfig['PATH'])
+                else:
+                    matchItems=_matchThings(keyWord)
+                debugOut('sMatch',matchItems,STDOUT=STDOUT)
+                if len(matchItems)>1:#不止一个匹配项
+                    shortest=min(matchItems, key=len)#最短匹配项
+                    index=0
+                    FINISHED=False
+                    debugOut('shortest ',shortest,STDOUT=STDOUT)
+                    matchItems.remove(shortest)
+                    for i in shortest:
+                        for item in matchItems:
+                            #debugOut('tab',item[index]+' vs shortest '+i)
+                            if item[index]!=i:#某项目该位字符不匹配
+                                FINISHED=True
+                        if not FINISHED:
+                            index+=1
+                        else:
+                            break
+                    matchItems.append(shortest)
+                    #consoleOut('tab',index)
+                    if index>len(keyWord):
+                        PreExcute=shortest[len(keyWord):index]
+                        #consoleOut('pre',PreExcute)
+                    else:
+                        cursor.hide()
+                        consoleOut('',','.join(matchItems),'SPACE','white',STDOUT=STDOUT)
+                        say(_x+indexX,_y,'')#把光标移回命令行
+                        cursor.show()
+                elif len(matchItems)==1:
+                    
+                    result=matchItems.pop()
+                    #---支持忽略.py后缀
+                    if len(result)>3:
+                        #consoleOut('a',result[:-3])
+                        #consoleOut('b',result[-3:])
+                        resultWithoutPy=result[:-3]+result[-3:].replace('.py','')#把文件结尾的.py去掉
+                        #consoleOut('c',resultWithoutPy)
+                    else:
+                        resultWithoutPy=result
+                    if len(resultWithoutPy)>len(keyWord):
+                        PreExcute=resultWithoutPy[len(keyWord):]
+                    else:
+                        PreExcute=result[len(keyWord):]
+        elif whiteList:
+            try:
+                if selection in keyList:
+                    if indexX<limitLength:
+                        say(_x+indexX,_y,selection,color)
+                        chrWidth=getWidth(selection)
+                        indexX+=chrWidth
+                        retStr+=selection
+                        if selection!=' ':
+                            keyWord+=selection#tab补全用
+                        else:
+                            keyWords.append(keyWord)
+                            keyWord=''
+                    else:#过长
+                        
+                        say(_x,_y,retStr,_red)
+                        time.sleep(0.01)
+                        say(_x,_y,retStr,color)
+            except Exception as e:
+                consoleOut('Input Error1',str(e),red,yellow)
+                pass
+        else:
+            try:
+                if indexX<limitLength:
+                    chrWidth=getWidth(selection)
+                    
+                    if hideChar=='$n0p':
+                        say(_x+indexX,_y,selection,color)
+                    else:
+                        say(_x+indexX,_y,hideChar[0]*chrWidth,color)
+                    
+                    indexX+=chrWidth
+                    retStr+=selection
+                    if selection!=' ':
+                        keyWord+=selection#tab补全用
+                    else:
+                        keyWords.append(keyWord)
+                        keyWord=''
+                else:
+                    PreExcute='\\'+'\n'+selection
+                    #SAY(_x,_y,retStr,_red)
+                    #time.sleep(0.01)
+                    #SAY(_x,_y,retStr,color)
+            except Exception as e:
+                consoleOut('Input Error2',str(e),red,yellow)
+                pass
+def _matchThings(target,matchWords='$n0p',matchPaths='$n0p',matchCurrentPath=True,matchVars=True,matchCommand=True,matchPATH=True):
+    allItems=list()
+    accessItems=list()
+    searchPaths=list()
+    if matchPATH:
+        matchCurrentPath=True
+    def searchDir(searchPaths,onlyFile=False):
+        retItems=list()
+        for path in searchPaths:
+                try:
+                    if realTargetPath==path:
+                        for item in os.listdir(path):
+                            if os.path.isfile(path+item):
+                                retItems.append(targetPath+item)
+                            else:
+                                #if srcPath+_file==os.path.join(root,_file):#在top目录下
+                                if not onlyFile:
+                                    retItems.append(targetPath+item+'/')
+                    else:
+                        for item in os.listdir(path):
+                            #debugOut('Try isfile',path+item)
+                            if os.path.isfile(path+item):
+                                retItems.append(item)
+                            else:
+                                #if srcPath+_file==os.path.join(root,_file):#在top目录下
+                                if not onlyFile:
+                                    retItems.append(item+'/')
+                    #consoleOut('matchThings',path+' LOAD FINISHED.','green','white')
+                except Exception as e:
+                    debugOut('matchThings',path+' LOAD FAILED.'+str(e),'red')
+        return retItems
+    targetName,targetPath=splitPath(target)
+    if targetName!='$n0p':
+        if targetName[0]=='$' and matchVars:
+            for varName in __PublicVarPool__:
+                try:
+                    if targetName[1]=='(':
+                        allItems.append('$('+varName+')')
+                    else:
+                        allItems.append('$'+varName)
+                except:
+                    allItems.append('$'+varName)
+        else:
+            if matchCurrentPath:
+                if targetPath!='':
+                    realTargetPath=realPath(targetPath)
+                    searchPaths.append(realTargetPath)#path位置
+                    targetName=target#保持原目标
+                else:
+                    realTargetPath=''
+                    searchPaths.append(__SHELLPATH__)#默认shell当前位置
+            if matchPaths!='$n0p':
+                searchPaths+=matchPaths
+            allItems+=searchDir(searchPaths)
+            if matchWords!='$n0p':
+                allItems+=matchWords
+    #-----MATCH---------------------------
+        #debugOut('allItems',allItems)
+        newItems=[]
+        for i in allItems:
+            if i not in newItems:
+                newItems.append(i)
+        allItems=newItems
+        index=0
+        for i in targetName:
+            itemIndex=0
+            for item in allItems:
+                _char=_getPart(index,item)
+                if _char=='$n0p':#不够长
+                    allItems[itemIndex]=''
+                else:
+                    if i!=_char:#不符合
+                        allItems[itemIndex]=''
+                itemIndex+=1
+            index+=1
+        for item in allItems:
+            if item!='':
+                accessItems.append(item)
+        return accessItems
+    else:
+        return list()
+def realPath(_path,isFile=False):
+    retPath=legalPath(_path)
+    if retPath=='$n0p/':#当前shell目录
+        targetPath=__SHELLPATH__
+    elif retPath[0]=='/':#绝对路径
+        targetPath=retPath
+    else:#相对路径
+        targetPath=__SHELLPATH__+retPath
+    if isFile:
+        return targetPath[:-1]
+    else:
+        return targetPath
+def legalPath(_path):
+    if _path!='':
+        if _path[-1]!=__dirChar__:
+            _path+=__dirChar__
+        _path=_path.replace('N0:/',__WORKPATH__)#N0目录转linux目录
+        if len(_path)>2:
+            _path=_path[:2]+_path[2:].replace('./','')#去当前目录
+    return _path
 def divide(string,TheChar,delESC=False):
     RIGHT=False
     right=""
@@ -640,6 +1041,14 @@ def divide(string,TheChar,delESC=False):
         return left,'$n0p'
     else:
         return left,right
+def _getPart(id,parts,default='$n0p'):
+    try:
+        if parts[id]=='':
+            return '$n0p'
+        else:
+            return parts[id]
+    except:
+        return default
 def divideStrByRealLength(string,targetLength):
     real_length=0
     retStr=''
@@ -1147,6 +1556,12 @@ def newArgv(offset):
         else:
             return retParts
         index+=1
+def pause(string='pause...',color=yellow,STDOUT='$n0p'):
+    if STDOUT=='$n0p':
+        STDOUT=__SYSTEM_VARS__['__STDOUT__']
+    consoleOut('',string,'SPACE',color,STDOUT=STDOUT)
+    ret=_consoleWin.getch()
+    return ret
 def initGraphic(window="$n0p"):
     global _consoleWin,__limitX__,__limitY__,__GRAPHIC__
     if not __GRAPHIC__:
@@ -1180,4 +1595,5 @@ def endGraphic():
         _consoleWin.keypad(False)
         curses.echo()
         curses.endwin()
+        sys.stdout=__DEFAULT_STDOUT__
     cursor.show()
